@@ -148,7 +148,7 @@ def fillmissing(ar, value):
     See :`ismissing` for the definition of missing values.
     '''
     dtype = vaex.dtype_of(ar)
-    if dtype == str:
+    if dtype.is_arrow:
         return pc.fill_null(ar, value)
     ar = ar if not isinstance(ar, column.Column) else ar.to_numpy()
     mask = ismissing(ar)
@@ -270,10 +270,21 @@ def notna(x):
 
 ########## datetime operations ##########
 
+@register_function(on_expression=False)
+def scalar_datetime(datetime_str):
+    return np.datetime64(datetime_str)
+
+
+@register_function(on_expression=False)
+def scalar_timedelta(amount, unit):
+    return np.timedelta64(amount, unit)
+
 
 def _pandas_dt_fix(x):
     # see https://github.com/pandas-dev/pandas/issues/23276
     # not sure which version this is fixed in
+    if isinstance(x, pa.lib.TimestampArray):
+        return x.to_pandas()
     if not x.flags['WRITEABLE']:
         x = x.copy()
     return x
@@ -281,7 +292,8 @@ def _pandas_dt_fix(x):
 def _to_pandas_series(x):
     import pandas as pd
     # pandas seems to eager to infer dtype=object for v1.2
-    return pd.Series(_pandas_dt_fix(x), dtype=x.dtype)
+    x = _pandas_dt_fix(x)
+    return pd.Series(x, dtype=x.dtype)
 
 @register_function(scope='dt', as_property=True)
 def dt_date(x):
@@ -1922,7 +1934,7 @@ def str_rsplit(x, pattern=None, max_splits=-1):
     if pattern is None:
         return pc.utf8_split_whitespace(x, reverse=True, max_splits=max_splits)
     else:
-        return pc.split_pattern(x, reverse=True, max_splits=max_splits)
+        return pc.split_pattern(x, pattern=pattern, reverse=True, max_splits=max_splits)
 
 
 @register_function(scope='str')
@@ -1936,6 +1948,44 @@ def str_split(x, pattern=None, max_splits=-1):
         return pc.split_pattern(x, pattern=pattern, max_splits=max_splits)
 
 
+@register_function(scope='str')
+@auto_str_unwrap
+def str_extract_regex(x, pattern):
+    """Extract substrings defined by a regular expression using Apache Arrow (Google RE2 library).
+
+    :param str pattern: A regular expression which needs to contain named capture groups, e.g. ‘letter’ and ‘digit’ for
+                        the regular expression '(?P<letter>[ab])(?P<digit>\\d)'.
+    :returns: an expression containing a struct with field names corresponding to capture group identifiers.
+
+    Example:
+
+    >>> import vaex
+    >>> email = ["foo@bar.org", "bar@foo.org", "open@source.org", "invalid@address.com"]
+    >>> df = vaex.from_arrays(email=email)
+    >>> df
+    #  email
+    0  foo@bar.org
+    1  bar@foo.org
+    2  open@source.org
+    3  invalid@address.com
+
+    >>> pattern = "(?P<name>.*)@(?P<address>.*)\.org"
+    >>> df.email.str.extract_regex(pattern=pattern)
+    Expression = str_extract_regex(email, pattern='(?P<name>.*)@(?P<addres...
+    Length: 4 dtype: struct<name: string, address: string> (expression)
+    -------------------------------------------------------------------
+    0      {'name': 'foo', 'address': 'bar'}
+    1      {'name': 'bar', 'address': 'foo'}
+    2  {'name': 'open', 'address': 'source'}
+    3                                     --
+
+    """
+
+    if not isinstance(x, vaex.array_types.supported_arrow_array_types):
+        x = pa.array(x)
+
+    return pc.extract_regex(x, pattern=pattern)
+
 
 @register_function(scope='str')
 @auto_str_unwrap
@@ -2395,7 +2445,12 @@ def _ordinal_values(x, ordered_set):
     if not isinstance(x, vaex.array_types.supported_array_types) or isinstance(ordered_set, vaex.superutils.ordered_set_string):
         # sometimes the dtype can be object, but seen as an string array
         x = _to_string_sequence(x)
-    return ordered_set.map_ordinal(x)
+    else:
+        x = vaex.array_types.to_numpy(x)
+    indices = ordered_set.map_ordinal(x)
+    if np.ma.isMaskedArray(x):
+        indices[x.mask] = ordered_set.null_value
+    return indices
 
 @register_function()
 def _choose(ar, choices, default=None):
@@ -2433,10 +2488,12 @@ def _map(ar, value_to_index, choices, default_value=None, use_missing=False, axi
     else:
         ar = vaex.array_types.to_numpy(ar)
         indices = value_to_index.map_ordinal(ar) + 1
-        if np.ma.isMaskedArray(ar):
-            mask = np.ma.getmaskarray(ar)
-            indices[mask] = 1  # missing values are at offset 1 (see expression.map)
     values = choices.take(indices)
+    if np.ma.isMaskedArray(ar):
+        mask = np.ma.getmaskarray(ar).copy()
+        # also mask out the missing (which had -1 and was moved to 0)
+        mask = mask | (indices == 0)
+        values = np.ma.array(values, mask=mask)
     if flatten:
         values = wrapper(values)
     return values
@@ -2455,13 +2512,25 @@ def _astype(x, dtype):
             # arrow does not support timestamp to int/float, so we use numpy
             y = x.to_numpy().astype(dtype)
             return y
+        if pa.types.is_duration(x.type):
+            # for consistency with timedelta, use numpy for this for now
+            y = x.to_numpy().astype(dtype)
+            return y
 
-        if dtype.startswith('datetime64'):  # parse dtype
-            if len(dtype) > len('datetime64'):
-                units = dtype[len('datetime64')+1:-1]
-            else:
-                units = 'ns'
-            dtype = pa.timestamp(units)
+        if (dtype.startswith('datetime64') | dtype.startswith('timedelta64')):
+            if dtype.startswith('datetime64'):  # parse dtype
+                if len(dtype) > len('datetime64'):
+                    units = dtype[len('datetime64')+1:-1]
+                else:
+                    units = 'ns'
+                dtype = pa.timestamp(units)
+
+            else:  # parse dtype
+                if len(dtype) > len('timedelta64'):
+                    units = dtype[len('timedelta64')+1:-1]
+                else:
+                    units = 'ns'
+                dtype = pa.duration(units)
 
         y = x.cast(dtype, safe=False)
         return y
@@ -2684,3 +2753,18 @@ def stack(arrays, strict=False):
 def getitem(ar, item):
     slicer = (slice(None), item)
     return ar.__getitem__(slicer)
+
+
+@register_function()
+def dot_product(a, b):
+    '''Compute the dot product between `a` and `b`.
+
+    :param a: A list of Expressions or a list of values (e.g. a vector)
+    :param b: A list of Expressions or a list of values (e.g. a vector)
+    :return: Vaex expression
+    '''
+    assert len(a) == len(b), 'The lenghts of `a` and `b` must be the same.'
+    result = a[0] * b[0]
+    for an, bn in zip(a[1:], b[1:]):
+        result += an * bn
+    return result
